
Object.defineProperty(exports, '__esModule', { value: true });
const fs_1 = require('fs');
const path_1 = require('path');
const minimist = require('minimist');
const types_1 = require('../types/types');
const __1 = require('..');

function filterFiles(files, baseExt = '') {
	const result = [];
	if (files.length === 0) files.push(process.cwd());
	for (let i = files.length - 1; i >= 0; i--) {
		const file = path_1.resolve(files[i]);
		if (fs_1.existsSync(file)) {
			try {
				let dirContent = fs_1.readdirSync(file);
				if (dirContent.length > 0) {
					dirContent = dirContent
						.filter(content => content !== 'node_modules')
						.map(content => {
							return path_1.resolve(file, content);
						});
					const content = filterFiles(dirContent);
					result.push(...content);
				}
			} catch (error) {
				if (error.code === 'ENOTDIR') {
					if (baseExt) {
						if (!new RegExp(`\\.?${baseExt}$`).test(file)) continue;
					} else if (!/\.d\.ts$/.test(files[i])) continue;
					result.push(file);
				}
			}
		}
	}
	return result;
}
function cleanDtsCli(args) {
	const parsedArgs = minimist(args.slice(2));
	let files = parsedArgs._;
	const options = {};
	if (parsedArgs.keepAll || parsedArgs.keepall)
		options.storeStrategy = types_1.Strategies.keepAll;
	else if (parsedArgs.keepPartial || parsedArgs.keeppartial)
		options.storeStrategy = types_1.Strategies.keepPartial;
	options.all = parsedArgs.all === true ? '' : parsedArgs.all;
	options.newName = parsedArgs.newName || parsedArgs.newname || '';
	options.ext = parsedArgs.ext || '';
	options.outputDir = parsedArgs.outputDir || parsedArgs.outputdir || '';
	options.baseExt = parsedArgs.baseExt || parsedArgs.baseext || '';
	options.includePrivates = parsedArgs.privates || false;
	options.excludeProtected = parsedArgs.protected || false;
	if (parsedArgs.mod) options.mod = parsedArgs.mod.split(',');
	if (parsedArgs.restrict) options.restrict = parsedArgs.restrict;
	files = filterFiles(files, options.baseExt);
	if (files.length === 0) {
		files.push(process.cwd());
	}
	let resultStr = '';
	const resultArr = [];
	files.forEach(file => {
		options.file = file;
		if (parsedArgs.sync) {
			const result = __1.cleanDtsSync(options);
			if (result) resultStr += result;
		} else {
			resultArr.push(__1.cleanDts(options).catch(err => console.log(err)));
		}
	});
	if (resultStr) return resultStr;
	if (resultArr.length > 0) return resultArr;
	
}
exports.cleanDtsCli = cleanDtsCli;
