declare module 'clean-dts' {
	/**
	 * The available options for Clean Dts
	 */
	export type Options = {
		/**
		 * Determmine how the modules should be stored if imported by other modules.
		 */
		storeStrategy?: Strategies;
		mod?: string | string[];
		restrict?: boolean;
		/** Change all module names to the value provided here */
		all?: string;
		/** The file to parse through (name and location) */
		file?: string;
		/** change Name (only name not location) */
		newName?: string;
		/** add extension type */
		ext?: string;
		/** the output location (only directory not name) */
		outputDir?: string;
		/** base Extension */
		baseExt?: string;
		includePrivates?: boolean;
		excludeProtected?: boolean;
	};
	export enum Strategies {
		/** Remove modules which are referenced from main */
		none = 0,
		/**
		 * Only keep modules which was partially imported into other modules.
		 * If all exports were imported then module will be removed, unless
		 * there are declared variables.
		 */
		keepPartial = 1,
		/**
		 * Will keep all modules.
		 */
		keepAll = 2,
	}

	/**
	 * Main Synchronous function that ensure that d.ts files appear clean.
	 * @param options An Options object.
	 * @param text Optional text to parse through.
	 */
	export function cleanDtsSync(options?: Options, text?: string): string | void;
	/**
	 * Main Asynchronous function that ensure that d.ts files appear clean.
	 * @param opts An Options object.
	 * @param text Optional text to parse through.
	 */
	export function cleanDts(options?: Options, text?: string): Promise<string | void>;
}
