
Object.defineProperty(exports, '__esModule', { value: true });
const fs_1 = require('fs');
const path_1 = require('path');
const deconstruct_1 = require('./lib/deconstruct');
const reconstruct_1 = require('./lib/reconstruct');

function promise(func, ...params) {
	return new Promise((res, rej) => {
		return func(...params, (err, data) => {
			if (err) return rej(err);
			return res(data.toString());
		});
	});
}
function onlyErrorPromise(func, ...params) {
	return new Promise((res, rej) => {
		return func(...params, err => {
			if (err) rej(err);
			res();
		});
	});
}
function getDestination(location, options) {
	const parsed = path_1.parse(location || '');
	if (options.newName) parsed.name = options.newName;
	if (options.outputDir) parsed.dir = options.outputDir;
	if (options.ext) {
		let reg;
		if (options.baseExt) {
			reg = new RegExp(`\\.?${options.baseExt}$`);
			parsed.name = parsed.base.replace(reg, '');
		}
		parsed.ext = options.ext;
	}
	parsed.base = parsed.name + (parsed.ext.charAt(0) === '.' ? parsed.ext : `.${parsed.ext}`);
	const dir = path_1.format(parsed);
	if (dir === '.') return '';
	return path_1.resolve(dir);
}
function CleanError(msg, error) {
	this.msg = msg;
	this.error = error;
	this.stack = new Error().stack;
	this.name = 'cleanError';
}
function cleanDtsSync(options = {}, text = '') {
	let src = text;
	const opts = options == null ? {} : options;
	if (!text && opts.file) {
		try {
			src = fs_1.readFileSync(opts.file).toString();
		} catch (error) {
			throw new CleanError(
				`An error has occured attempting to read file ${opts.file}.`,
				error,
			);
		}
	}
	const parsed = deconstruct_1.parseCode(src, opts);
	let result = text;
	if (parsed) {
		const res = reconstruct_1.reconstruct(parsed, opts);
		if (res) result = res;
	}
	const store = getDestination(opts.file, opts);
	if (!store) return result;
	const storeParse = path_1.parse(store);
	try {
		fs_1.mkdirSync(storeParse.dir);
	} catch (error) {
		if (error.code !== 'EEXIST')
			throw new CleanError(`an error occured creating folder ${storeParse.dir}`, error);
	}
	try {
		fs_1.writeFileSync(store, result);
	} catch (error) {
		throw new CleanError(`an error has occured attempting to write to ${store}.`, error);
	}
	
}
exports.cleanDtsSync = cleanDtsSync;
async function cleanDts(options = {}, text = '') {
	let src = text;
	const opts = options == null ? {} : options;
	if (!text && opts.file) {
		try {
			src = await promise(fs_1.readFile, opts.file);
		} catch (error) {
			return Promise.reject(
				new CleanError(`An error has occured attempting to read file ${opts.file}.`, error),
			);
		}
	}
	const parsed = deconstruct_1.parseCode(src, opts);
	let result = text;
	if (parsed) {
		const res = reconstruct_1.reconstruct(parsed, opts);
		if (res) result = res;
	}
	const store = getDestination(opts.file, opts);
	if (!store) return Promise.resolve(result);
	const storeParse = path_1.parse(store);
	try {
		await onlyErrorPromise(fs_1.mkdir, storeParse.dir);
	} catch (error) {
		if (error.code !== 'EEXIST')
			return Promise.reject(
				new CleanError(`an error occured creating folder ${storeParse.dir}`, error),
			);
	}
	try {
		await onlyErrorPromise(fs_1.writeFile, store, result);
	} catch (error) {
		Promise.reject(
			new CleanError(`an error has occured attempting to write to ${store}.`, error),
		);
	}
	return Promise.resolve();
}
exports.cleanDts = cleanDts;
