
Object.defineProperty(exports, '__esModule', { value: true });
const fs_1 = require('fs');
const types_1 = require('../types/types');

function getCode(mod, modules, incl = false) {
	if (!mod) return false;
	let c = mod.code;
	const reg = /\s*?([ \t]+)?\n--replace([^\s]*?)=(.*?)\[(all|grp)?(im|ex)\]/;
	const importGrp = {};
	let grpNames = {};
	const exportChk = [];
	const addImport = (from, mbr) => {
		if (!importGrp[from]) importGrp[from] = [mbr];
		else importGrp[from].push(mbr);
	};
	const addGrp = (name, from) => {
		if (grpNames[from]) grpNames[from].push(name);
		else grpNames[from] = [name];
	};
	const listToString = (mem, val) => {
		if (mem == null) return val.join(', ');
		let tmpText = '';
		val.forEach(nme => {
			if (tmpText !== '') tmpText += ', ';
			if (mem[nme] && mem[nme].as) {
				tmpText += `${mem[nme].name} as `;
			}
			tmpText += nme;
		});
		return tmpText;
	};
	const getGrp = () => {
		let res = '';
		const keys = Object.keys(grpNames);
		keys.forEach(key => {
			const txt = listToString(mod.exports, grpNames[key]);
			res += `export { ${txt} }`;
			if (key) res += ` from '${key}'`;
			res += ';';
		});
		return `${res}\n`;
	};
	const addMember = mbr => {
		let result = '';
		const nme = mbr.as || mbr.name;
		let { comment = '' } = mbr;
		const { portType } = mbr;
		if (exportChk.indexOf(nme) > -1) return '';
		if (mbr.all) {
			const frm = mbr.from;
			if (!modules[frm] || !mbr.include) {
				result = `${comment}${portType}port *${mbr.as ? ` as ${nme}` : ''} from ${frm};`;
			} else if (mbr.include) {
				const code = getCode(modules[frm], modules, true);
				if (code) {
					if (mbr.as) {
						result = `${comment}${portType}port namespace ${mbr.as} {${code}}`;
					} else result = code;
				}
			}
		} else if (mbr.value) {
			if (mbr.value.comment) comment = mbr.value.comment;
			result = `${comment}${mbr.portType}port${mbr.value.prename.replace(/^\s*default/, '')}`;
			result = `${result.trimRight()} ${nme}`;
			if (mbr.value.type === types_1.codeType.curlyBracket) result += `{${mbr.value.code}}`;
			else result += mbr.value.code;
		} else if (nme && nme !== '*' && mbr.portType === 'ex') {
			result += `export ${mbr.name} from '${mbr.from}';`;
		}
		if (nme !== '*') exportChk.push(nme);
		return result;
	};
	const replace = res => {
		if (res) c = c.replace(/\n--replace([^\s]*?)=(.*?)\[(all|grp)?(im|ex)\]/, res);
		else c = c.replace(/\s*--replace([^\s]*?)=(.*?)\[(all|grp)?(im|ex)\]/, '');
	};
	while (reg.test(c)) {
		let result = '';
		const [, space, id, name, t, port] = reg.exec(c);
		grpNames = {};
		if (t === 'all') {
			if (port === 'im') result += `import '${name}';`;
			else {
				const newMod = modules[name];
				if (mod.includeArr && mod.includeArr.indexOf(name) > -1) {
					const res = getCode(newMod, modules, true);
					if (res) {
						result += /^[ \t]*\n/.test(res) ? res.replace(/^[ \t]*\n/, '') : res;
						replace(result);
						continue;
					}
				}
				if (newMod && newMod.newName) result += `export * from '${newMod.newName}'`;
				else result += `export * from '${name}';`;
			}
		} else if (t === 'grp') {
			const grp = mod.groups[id];
			if (port === 'im') {
				if (name && !modules[name]) {
					const tmpText = listToString(mod.imports, grp);
					result += `import { ${tmpText} } from '${name}';`;
				}
			} else {
				grp.forEach(exName => {
					if (
						!mod.exports[exName] ||
						(mod.exports[exName] && !mod.exports[exName].include && !incl) ||
						(mod.exports[exName] && mod.exports[exName].value === false)
					) {
						if (mod.exports[exName]) {
							addGrp(exName, mod.exports[exName].from);
						} else if (mod.imports[exName])
							addImport(mod.imports[exName].from, mod.imports[exName]);
					} else {
						const itm = mod.exports[exName];
						const res = addMember(itm);
						if (res) result += res;
					}
				});
				result = getGrp() + space + result;
			}
		} else if (mod.exports[name]) {
			result = addMember(mod.exports[name]);
		} else if (mod.imports[name]) {
			addImport(mod.imports[name].from, mod.imports[name]);
		}
		replace(result);
	}
	const keys = Object.keys(importGrp);
	if (keys.length > 0) {
		keys.forEach(key => {
			const lst = [];
			importGrp[key].forEach(imprt => {
				if (imprt.include) lst.push(imprt.as ? `${imprt.name} as ${imprt.as}` : imprt.name);
			});
			if (fs_1.lstat.length > 0) c = `import { ${lst.join(',')} } from ${key};`;
		});
	}
	return c;
}
function mustUse(mod, options) {
	const strat = options.storeStrategy ? options.storeStrategy : types_1.Strategies.none;
	if (mod.done) return false;
	if (options.restrict && !mod.isMod) return false;
	if (strat === types_1.Strategies.keepAll || (!mod.allused && !mod.used) || mod.done)
		return true;
	if (mod.isMod || (mod.hasDeclare && !mod.allused)) return true;
	if (strat === types_1.Strategies.none && (mod.used || mod.allused)) return false;
	if (mod.allused) return false;
	if (strat !== types_1.Strategies.keepPartial) return false;
	return true;
}
function cleanup(code, mods) {
	let result = code;
	const imprts = [];
	const exprts = [];
	const reg = /([ \t]*\/\*.*?\*\/\s*?)?([ \t]*)(im|ex)port *((({(.*?)}|(\*( *as *.*?)?)) ) *from )?['"](.*?)['"];[ \t]*[\n\r]*/;
	while (reg.test(result)) {
		const [main, comment, space, port, , , , complx, , , name, end] = reg.exec(result);
		result = result.replace(reg, '');
		if (mods.indexOf(name) === -1) {
			if (port === 'im') imprts.push(main);
			else exprts.push(main);
		} else if (port === 'ex' && complx) {
			const cSplit = complx.split(',');
			let str = '';
			cSplit.forEach(c => {
				if (/.*? *as *.*?/.test(c)) {
					if (str) str += ',';
					str += c;
				}
			});
			if (str) {
				str = `${comment || ''}${space}${port}port {${str}}`;
				if (name && mods.indexOf(name) === -1) str += ` from '${name}'`;
				str += `;${end || ''}`;
				exprts.push(str);
			}
		}
	}
	const reg2 = /[ \t]*(im|ex)port *[^ {}]*? *from *['"](.*)['"];[ \t]*[\r\n]*/;
	while (reg2.test(result)) {
		const [main, port, name] = reg2.exec(result);
		if (mods.indexOf(name) === -1) {
			if (port === 'ex') exprts.push(main);
			else imprts.push(main);
		}
		result = result.replace(reg2, '');
	}
	imprts.forEach(x => {
		result = x + result;
	});
	exprts.forEach(x => {
		result += x;
	});
	result = result.replace(reg2, '');
	return result;
}
function getName(name) {
	if (/[ \\\-/+=*~]/.test(name.trim())) return `'${name.trim()}'`;
	return name;
}
function reconstruct(modules, options = {}) {
	let result = '';
	const mods = { ...modules };
	const all = options.mod ? undefined : options.all === '' ? '/' : options.all;
	const keys = Object.keys(mods);
	const clearText = txt => {
		if (/[^\s]/.test(txt)) return txt;
		return '';
	};
	const getCodeCheck = (mod, key) => {
		if (
			mod.newName &&
			mods[mod.newName] &&
			(mods[mod.newName].newName === mod.newName || mods[mod.newName].done)
		)
			return '';
		let code = getCode(mod, mods);
		if (code) mods[key].done = true;
		else code = '';
		code = clearText(code);
		for (let i = 0, len = keys.length; i < len; i++) {
			if (keys[i] === key) continue;
			if (
				mods[keys[i]].newName === key ||
				(mod.newName && mod.newName === mods[keys[i]].newName)
			) {
				const c = getCode(mods[keys[i]], mods);
				if (c) {
					code += clearText(c);
					mods[keys[i]].done = true;
				}
			}
		}
		return code;
	};
	for (let i = 0, len = keys.length; i < len; i++) {
		const mod = mods[keys[i]];
		if (mustUse(mod, options)) {
			const code = getCodeCheck(mod, keys[i]);
			if (code && /\s?.\s?/.test(code)) {
				if (all == null) {
					const nme = mod.newName || mod.name;
					if (nme === '' || nme === '/') result += code;
					else result += `declare module ${getName(mod.newName || mod.name)} {${code}}\n`;
				} else result += `${code}\n`;
			}
		}
	}
	if (all) {
		result = cleanup(result, Object.keys(mods));
	}
	if (all && all !== '/') result = `declare module ${getName(all)} {${result}}`;
	return result;
}
exports.reconstruct = reconstruct;
